@@ -84,8 +84,7 @@ ph_add_text <- function( x, str, type = NULL, id_chr = NULL,
 
   xml_add_child(current_p, new_node, .where = where_ )
 
-  slide$save()
-  x$slide$update_slide(x$cursor)
+  slide$fortify_id()$save()
 
   x
 }
@@ -138,8 +137,7 @@ ph_add_par <- function( x, type = NULL, id_chr = NULL, level = 1 ){
   }
 
 
-  slide$save()
-  x$slide$update_slide(x$cursor)
+  slide$fortify_id()$save()
 
   x
 }
@@ -201,8 +199,7 @@ ph_add_fpar <- function( x, value, type = "body", id_chr = NULL, level = 1, par_
     xml_add_child(current_p, node )
   }
 
-  slide$save()
-  x$slide$update_slide(x$cursor)
+  slide$fortify_id()$save()
 
   x
 }
